/*
 * Copyright (c) 2014 Nokia Solutions and Networks
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */


#include <linux/init.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/device.h>
#include <linux/cdev.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/seq_file.h>
#include <asm/uaccess.h>
#include <mod_utils.h>
#include "linux/fzm_ioctls.h"
#include "linux/cpld-usr.h"
#include <linux/delay.h>
#include <linux/version.h>
#include <linux/interrupt.h>
#include <linux/of_irq.h>
#include <linux/platform_device.h>
#include "cpld_internal.h"

/* Bursts are currently broken on the CPLD.  They sometimes return the same register twice */
/* #define USE_BURST */
#define NR_REAL_CPLD_IRQS 16

struct cpld_ic_reg {
   u16 bits;
};

struct cpld_ic {
   struct device *dev;
   struct irq_domain *domain;
   struct mutex lock;
   struct cpld_ic_reg mask;
   struct cpld_ic_reg oldmask;
   int virq;
};

static struct cpld_ic cpld_ic;


int cpld_ic_raw_status(unsigned int virq)
{
   unsigned bit;
   u8 reg;
   struct irq_data *irqd = irq_get_irq_data(virq);

   if (!irqd)
      return -EINVAL;

   if (irqd->domain != cpld_ic.domain)
      return -EXDEV;

   bit = irqd_to_hwirq(irqd);

   if (bit >= 8)
      reg = readb_cpld_swstat2();
   else
      reg = readb_cpld_swstat();

   return (reg & BIT(bit & 7)) ? 1 : 0;
}
EXPORT_SYMBOL(cpld_ic_raw_status);

/* cpld ic read / write functions */
static int cpld_ic_read_irqr(struct cpld_ic_reg *reg)
{
#ifdef USE_BURST
   u8 val[4];
   int err;

   /* read swints (0x3c) and swints2 (0x48) in one go */
   err = fzm_cpldspi_read(FZM_CPLDSPI_CPLD,
                          offsetof(struct cpld_regs, swints),
                          &val, sizeof(val));

   /* ignore 0x40 and 0x44 bytes */
   reg->bits = val[0] | (val[3] << 8);

   dev_dbg(cpld_ic.dev, "%s:- %04x\n", __func__, irqr[0]);
#else
   u8 val[2];
   int err;

   err = fzm_cpldspi_read(FZM_CPLDSPI_CPLD,
                          offsetof(struct cpld_regs, swints),
                          &val[0], sizeof(u8));
   if (err)
      goto out;

   err = fzm_cpldspi_read(FZM_CPLDSPI_CPLD,
                          offsetof(struct cpld_regs, swints2),
                          &val[1], sizeof(u8));
   if (err)
      goto out;

   reg->bits = val[0] | (val[1] << 8);

   dev_dbg(cpld_ic.dev, "%s:- %04x\n", __func__, reg->bits);

out:
#endif

   return err;
}

static int cpld_ic_write_irqr(struct cpld_ic_reg *reg)
{
#ifdef USE_BURST
   u8 val[4];

   val[0] = reg->bits & 0xff;             /* swints */
   val[1] = cpld_ic.oldmask.bits & 0xff;  /* swintm */
   val[2] = 0;                            /* swstat2 */
   val[3] = reg->bits >> 8;               /* swints2 */

   dev_dbg(cpld_ic.dev, "%s:- %04x\n", __func__, reg->bits);

   return fzm_cpldspi_write(FZM_CPLDSPI_CPLD,
                            offsetof(struct cpld_regs, swints),
                            &val, sizeof(val));
#else
   u8 val;
   int err;

   dev_dbg(cpld_ic.dev, "%s:- %04x\n", __func__, reg->bits);

   val = reg->bits;
   err = fzm_cpldspi_write(FZM_CPLDSPI_CPLD,
                           offsetof(struct cpld_regs, swints),
                           &val, sizeof(u8));
   if (err)
      goto out;

   val = reg->bits >> 8;
   err = fzm_cpldspi_write(FZM_CPLDSPI_CPLD,
                           offsetof(struct cpld_regs, swints2),
                           &val, sizeof(u8));
out:
   return err;
#endif
}

static int cpld_ic_write_maskr(void)
{
#ifdef USE_BURST
   u8 val[4];

   val[0] = cpld_ic.mask.bits & 0xff;     /* swintm */
   val[1] = 0;                            /* swstat2 */
   val[2] = 0;                            /* swints2 */
   val[3] = cpld_ic.mask.bits >> 8;       /* swintm2 */

   dev_dbg(cpld_ic.dev, "%s:- %04x\n", __func__, cpld_ic.mask.bits);

   return fzm_cpldspi_write(FZM_CPLDSPI_CPLD,
                            offsetof(struct cpld_regs, swintm),
                            &val, sizeof(val));
#else
   u8 val;
   int err;

   dev_dbg(cpld_ic.dev, "%s:- %04x\n", __func__, cpld_ic.mask.bits);

   val = cpld_ic.mask.bits;
   err = fzm_cpldspi_write(FZM_CPLDSPI_CPLD,
                           offsetof(struct cpld_regs, swintm),
                           &val, sizeof(u8));
   if (err)
      goto out;

   val = cpld_ic.mask.bits >> 8;
   err = fzm_cpldspi_write(FZM_CPLDSPI_CPLD,
                           offsetof(struct cpld_regs, swintm2),
                           &val, sizeof(u8));
out:
   return err;
#endif
}

static void cpld_irq_unmask(struct irq_data *d)
{
   cpld_ic.mask.bits |= BIT(irqd_to_hwirq(d));
}

static void cpld_irq_mask(struct irq_data *d)
{
   cpld_ic.mask.bits &= ~BIT(irqd_to_hwirq(d));
}

static void cpld_ic_lock(struct irq_data *data)
{
   mutex_lock(&cpld_ic.lock);
}

static void cpld_ic_sync_unlock(struct irq_data *data)
{
   if (cpld_ic.oldmask.bits != cpld_ic.mask.bits) {
      cpld_ic_write_maskr();
      cpld_ic.oldmask.bits = cpld_ic.mask.bits;
   }
   mutex_unlock(&cpld_ic.lock);
}

static struct irq_chip cpld_irq_chip = {
   .name = "CPLD",
   .irq_unmask = cpld_irq_unmask,
   .irq_mask   = cpld_irq_mask,
   .irq_disable = cpld_irq_mask,
   .irq_bus_lock = cpld_ic_lock,
   .irq_bus_sync_unlock = cpld_ic_sync_unlock,
};

static int cpld_irq_map(struct irq_domain *h, unsigned int virq, irq_hw_number_t hw)
{
   irq_set_chip_data(virq, h->host_data);
   irq_set_chip(virq, &cpld_irq_chip);
   irq_set_nested_thread(virq, 1);
   return 0;
}

static const struct irq_domain_ops cpld_ic_domain_ops = {
   .map = cpld_irq_map,
   .xlate = irq_domain_xlate_onecell,
};

/* Cpld Interrupt Handler */
static irqreturn_t cpld_cascade_thread(int irq, void *data)
{
   struct cpld_ic *ic = data;
   struct cpld_ic_reg irqr;

   cpld_ic_read_irqr(&irqr);
   irqr.bits &= ic->mask.bits;

   if (!irqr.bits) {
      /* spurious IRQ! */
      return IRQ_NONE;
   }

   do {
      /* ACK */
      cpld_ic_write_irqr(&irqr);

      while (irqr.bits) {
         unsigned bit = __ffs(irqr.bits);
         irqr.bits &= ~BIT(bit);

         dev_dbg(cpld_ic.dev, "%s:- hwirq %u\n", __func__, bit);

         handle_nested_irq(irq_find_mapping(ic->domain, bit));
      }

      /* BUG IN CPLD: It should retrigger edge after any ACK if bits are
       * still pending.  If any unmasked bits were still set after ACK, we
       * need to handle them now, else we will never get another IRQ edge */
      cpld_ic_read_irqr(&irqr);
      irqr.bits &= ic->mask.bits;
   } while (irqr.bits);

   return IRQ_HANDLED;
}

int cpld_ic_probe(struct device *dev)
{
   struct device_node *np = dev->of_node;
   int err;
   struct cpld_ic_reg irqr;

   cpld_ic.virq = irq_of_parse_and_map(np, 0);
   if (!cpld_ic.virq) {
      dev_err(dev, "irq_of_parse_and_map failed\n");
      return -ENODEV;
   }

   mutex_init(&cpld_ic.lock);
   cpld_ic.dev = dev;

   /* Mask the interrupts */
   cpld_ic_write_maskr();

   /* Clear the SWINTS reg */
   irqr.bits = 0xffff;
   cpld_ic_write_irqr(&irqr);

   cpld_ic.domain = irq_domain_add_linear(np, NR_REAL_CPLD_IRQS, &cpld_ic_domain_ops, &cpld_ic);
   if (!cpld_ic.domain) {
      dev_err(dev, "cpld int probe: failed to request irq domain\n");
      return -ENOMEM;
   }

   /* Register the interrupt handler */
   err = request_threaded_irq(cpld_ic.virq, NULL, cpld_cascade_thread, IRQF_ONESHOT, "cpld-usr", &cpld_ic);
   if (err) {
      dev_err(dev, "cpld int probe: request_threaded_irq failed for irq %d\n", cpld_ic.virq);
      goto out_irq_domain;
   }

   return 0;

out_irq_domain:
   irq_domain_remove(cpld_ic.domain);

   return err;
}

int cpld_ic_remove(struct device *dev)
{
   unsigned hwirq;

   free_irq(cpld_ic.virq, &cpld_ic);

   /* remove all irq_desc structure which use this irq_chip that is going away */
   for (hwirq=0; hwirq<NR_REAL_CPLD_IRQS; ++hwirq)
      irq_dispose_mapping(irq_find_mapping(cpld_ic.domain, hwirq));

   irq_domain_remove(cpld_ic.domain);

   return 0;
}
