/*
 * Copyright (c) 2011-2014 Nokia Solutions and Networks
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * @file fpga.c
 * @author Nokia Solutions and Networks
 * @remarks Copyright (C) 2011-2014 Nokia Solutions and Networks
 *
 * @par PURPOSE
 * Vespa AP FPGA module.
 *
 * @par REVISIONS
 * <pre>
 * Date       Who       CR         Description
 * ---------- --------- ---------- --------------------------------------------
 *  04/12/2013 yongxu               Initial creation from lsp_k2_driver
 * </pre>
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/device.h>
#include <linux/cdev.h>
#include <linux/interrupt.h>
#include <linux/seq_file.h>
#include <linux/seqlock.h>
#include <asm/uaccess.h>
#include "fzm_fpga.h"
#include "fpga_internal.h"
#include "fzm_fpga_eis.h"

#if 0
#include "fpga.h"
#include "fpga_internal.h"
#endif

struct fpga_usr {
   struct device *dev;
   struct cdev cdev;
   struct proc_dir_entry *proc;
   struct sysfs_dirent *sysfs_rp3;
   seqcount_t rp3_lock;
   u32 rp3_fault;
   struct sysfs_dirent *sysfs_refab;
   seqcount_t refab_lock;
   u32 refab_fault;
};

struct fpga_usr_file {
   struct device *dev;
   bool is_locked;
};

static struct fpga_usr fpga_usr;

#if 0
#define SYSFS_WORD_R(name, addr) \
   static ssize_t name##_show(struct device *dev, struct device_attribute *attr, char *buf) \
   { \
      u32 reg; \
      int err; \
      \
      err = fpga_read(dev->parent, (addr), &reg, FPGA_SPI_DEVID_N_A); \
      if (err) \
         return err; \
      \
      return snprintf(buf, PAGE_SIZE, "0x%08x\n", reg); \
   }

#define SYSFS_BIT_W(name, addr, bit) \
   static ssize_t name##_store(struct device *dev, struct device_attribute *attr, \
                               const char *buf, size_t n) \
   { \
      int err; \
      bool val; \
      \
      if (!n) \
         return -EINVAL; \
      \
      err = strtobool(buf, &val); \
      if (err) \
         return err; \
      \
      err = fpga_modify(dev->parent, (addr), ~(1<<(bit)), val?(1<<(bit)):0); \
      if (err) \
         return err; \
      \
      return n; \
   }

#define SYSFS_BIT_R(name, addr, bit) \
   static ssize_t name##_show(struct device *dev, struct device_attribute *attr, char *buf) \
   { \
      u32 reg; \
      int err; \
      \
      err = fpga_read(dev->parent, (addr), &reg); \
      if (err) \
         return err; \
      \
      return snprintf(buf, PAGE_SIZE, "%c\n", (reg&(1<<(bit)))?'1':'0'); \
   }

#define SYSFS_BIT_RW(name, addr, bit) \
   SYSFS_BIT_R(name, addr, bit) \
   SYSFS_BIT_W(name, addr, bit)
#endif

#define SYSFS_WORD_R(name, addr) \
   static ssize_t name##_show(struct device *dev, struct device_attribute *attr, char *buf) \
   { \
      u32 reg; \
      int err; \
      \
      err = fpga_read(dev->parent, (addr), &reg, FPGA_SPI_DEVID_N_A); \
      if (err) \
         return err; \
      \
      return snprintf(buf, PAGE_SIZE, "0x%08x\n", reg); \
   }

#define SYSFS_WORD_W(name, addr) \
   static ssize_t name##_store(struct device *dev, struct device_attribute *attr, char *buf) \
   { \
      long reg; \
      int err; \
      \
      err = kstrtol(buf, 0, &reg); \
      if (err) \
         return err; \
      \
      err = fpga_write(dev->parent, (addr), (u32)reg, FPGA_SPI_DEVID_N_A); \
      if (err) \
         return err; \
      \
      return 0; \
   }

#define SYSFS_BIT_W(name, addr, bit) \
   static ssize_t name##_store(struct device *dev, struct device_attribute *attr, \
                               const char *buf, size_t n) \
   { \
      int err; \
      bool val; \
      \
      if (!n) \
         return -EINVAL; \
      \
      err = strtobool(buf, &val); \
      if (err) \
         return err; \
      \
      err = fpga_modify(dev->parent, (addr), (1<<(bit)), \
                        val?(1<<(bit)):0, FPGA_SPI_DEVID_N_A ); \
      if (err) \
         return err; \
      \
      return n; \
   }

#define SYSFS_BIT_R(name, addr, bit) \
   static ssize_t name##_show(struct device *dev, struct device_attribute *attr, char *buf) \
   { \
      u32 reg; \
      int err; \
      \
      err = fpga_read(dev->parent, (addr), &reg, FPGA_SPI_DEVID_N_A); \
      if (err) \
         return err; \
      \
      return snprintf(buf, PAGE_SIZE, "%c\n", (reg&(1<<(bit)))?'1':'0'); \
   }

#define SYSFS_BIT_RW(name, addr, bit) \
   SYSFS_BIT_R(name, addr, bit) \
   SYSFS_BIT_W(name, addr, bit)

#define SYSFS_WORD_RW(name, addr) \
   SYSFS_WORD_R(name, addr) \
   SYSFS_WORD_W(name, addr)

SYSFS_WORD_R(version, 0x00000000)

static ssize_t rp3_alarm_show(struct device *dev, struct device_attribute *attr, char *buf)
{
   int len = 0;
   unsigned int seq;
   do {
      seq = read_seqcount_begin(&fpga_usr.rp3_lock);
      len = sprintf(buf, "%x\n", fpga_usr.rp3_fault);
   } while (read_seqcount_retry(&fpga_usr.rp3_lock, seq));

   return len;
}

static ssize_t refab_alarm_show(struct device *dev, struct device_attribute *attr, char *buf)
{
   int len = 0;
   unsigned int seq;
   do {
      seq = read_seqcount_begin(&fpga_usr.refab_lock);
      len = sprintf(buf, "%x\n", fpga_usr.refab_fault);
   } while (read_seqcount_retry(&fpga_usr.refab_lock, seq));

   return len;
}

void update_rp3_alarm(u32 fault)
{
   write_seqcount_begin(&fpga_usr.rp3_lock);
   fpga_usr.rp3_fault = fault;
   write_seqcount_end(&fpga_usr.rp3_lock);
#ifdef FPGA_ALARM_TEST
   printk("rp3 fault updated with value: %x\n", fpga_usr.rp3_fault);
#endif
   sysfs_notify_dirent(fpga_usr.sysfs_rp3);

   return;
}

void update_refab_alarm(u32 fault)
{
   write_seqcount_begin(&fpga_usr.refab_lock);
   fpga_usr.refab_fault = fault;
   write_seqcount_end(&fpga_usr.refab_lock);
#ifdef FPGA_ALARM_TEST
   printk("refab fault updated with value: %x\n", fpga_usr.refab_fault);
#endif
   sysfs_notify_dirent(fpga_usr.sysfs_refab);

   return;
}

static struct device_attribute fpga_usr_attrs[] = {
   __ATTR_RO(version),
   __ATTR_RO(rp3_alarm),
   __ATTR_RO(refab_alarm),
   __ATTR_NULL,
};


static ssize_t fpga_usr_read(struct file *file, char __user *usrbuf, size_t len, loff_t *poff)
{
   struct fpga_usr_file *f = file->private_data;
   size_t rem;
   size_t off = *poff;
   u32 regs[256], *reg;
   int err;

   /* ensure size and alignment */
   if (off > FPGA_SIZE || (off & 3) || (len & 3)) {
      return -EFAULT;
   }

   if (off + len > FPGA_SIZE) {
      len = FPGA_SIZE - off;
   }

   if (len > sizeof(regs)) {
      len = sizeof(regs);
   }

   rem = len >> 2;
   reg = regs;

   /* empty read or EOF */
   if (!rem)
      return 0;

   if (!f->is_locked) {
      if (fpga_lock_interruptible(f->dev)) {
         return -ERESTARTSYS;
      }
   }

   err = fpga_read_block_unlocked(f->dev, off, reg, FPGA_SPI_DEVID_N_A, rem);
   if (err)
      goto out_unlock;

   if (copy_to_user(usrbuf, regs, len)) {
      err = -EFAULT;
      goto out_unlock;
   }

   *poff = off + len;
   err = len;

out_unlock:
   if (!f->is_locked)
      fpga_unlock(f->dev);

   return err;
}

static ssize_t fpga_usr_write(struct file *file, const char __user *usrbuf, size_t len, loff_t *poff)
{
   struct fpga_usr_file *f = file->private_data;
   size_t rem;
   size_t off = *poff;
   u32 regs[256], *reg;
   int err;

   /* ensure size and alignment */
   if (off > FPGA_SIZE || (off & 3) || (len & 3)) {
      return -EFAULT;
   }

   if (off + len > FPGA_SIZE) {
      len = FPGA_SIZE - off;
   }

   if (len > sizeof(regs)) {
      len = sizeof(regs);
   }

   if (copy_from_user(regs, usrbuf, len)) {
      return -EFAULT;
   }

   rem = len >> 2;
   reg = regs;

   /* empty write or EOF */
   if (!rem)
      return 0;

   if (!f->is_locked) {
      if (fpga_lock_interruptible(f->dev)) {
         return -ERESTARTSYS;
      }
   }

   err = fpga_write_block_unlocked(f->dev, off, reg, FPGA_SPI_DEVID_N_A, rem);
   if (err)
      goto out_unlock;

   *poff = off + len;
   err = len;

out_unlock:
   if (!f->is_locked)
      fpga_unlock(f->dev);

   return err;
}

static int fpga_usr_flock(struct file *file, int cmd, struct file_lock *fl)
{
   int err;
   struct fpga_usr_file *f = file->private_data;

   switch (fl->fl_type) {
   case F_RDLCK:
   case F_WRLCK:
      if (IS_SETLKW(cmd)) {
         if (fpga_lock_interruptible(f->dev)) {
            return -ERESTARTSYS;
         }
      } else if (!fpga_trylock(f->dev)) {
         return -EWOULDBLOCK;
      }

      err = flock_lock_file_wait(file, fl);
      if (err) {
         fpga_unlock(f->dev);
      } else {
         f->is_locked = true;
      }
      break;
   case F_UNLCK:
      fl->fl_flags |= FL_EXISTS;
      err = flock_lock_file_wait(file, fl);
      if (!err) {
         f->is_locked = false;
         fpga_unlock(f->dev);
      }
      break;
   default:
      return -EIO;
   }

   return err;
}

static int fpga_usr_open(struct inode *inode, struct file *file)
{
   struct fpga_usr *fpga = container_of(inode->i_cdev, struct fpga_usr, cdev);
   struct fpga_usr_file *f;

   f = kzalloc(sizeof(*f), GFP_KERNEL);
   if (!f)
      return -ENOMEM;

   f->dev = fpga->dev->parent;

   file->private_data = f;

   return 0;
}

static int fpga_usr_release(struct inode *inode, struct file *file)
{
   struct fpga_usr_file *f = file->private_data;

   kfree(f);

   return 0;
}

static long fpga_usr_ioctl(struct file *file, unsigned int cmd, unsigned long arg)
{
#ifdef FPGA_RF_TEST
   static u32 test_data = 0xA5A5A5A5;
#endif
   void __user *user_ptr = (void __user *)arg;
   struct fpga_usr_file *f = file->private_data;
   struct fpga_ioctl_rw rw;
   int err = 0;

   FUNC_TRACE;

   switch (cmd) {
   case FPGA_MASKED_WRITE:
      if (copy_from_user(&rw, user_ptr, sizeof(rw))) {
         printk("FPGA_MASKED_WRITE: copy_from_user failed\n");
         err = -EFAULT;
      } else {
#ifdef FPGA_RF_TEST
         test_data = (test_data & ~rw.mask) | (rw.value & rw.mask);
#else
         err = fpga_modify(f->dev, rw.addr, rw.mask, rw.value, rw.dev_id);
         if (err)
            return err;
#endif
      }
      break;
   default:
      printk("invalid ioctl: %u\n", cmd);
      err = -ENOTTY;
   }
   return err;
}

static struct file_operations fops = {
   .owner = THIS_MODULE,
   .open = fpga_usr_open,
   .release = fpga_usr_release,
   .unlocked_ioctl = fpga_usr_ioctl,
   .read = fpga_usr_read,
   .write = fpga_usr_write,
   .flock = fpga_usr_flock,
   .llseek = default_llseek,
};


static struct class fpga_class = {
      .name = "fpga_usr",
      .owner = THIS_MODULE,
      .dev_attrs = fpga_usr_attrs,
   };

/**
 * @internal
 * @brief Initializes the module.
 *
 * @return 0 on success.
 */
int fpga_usr_probe(struct device *pdev)
{
   int err;
   dev_t devt;

   err = alloc_chrdev_region(&devt, 0, 1, "fpga_usr");
   if (err) {
      dev_err(pdev, "alloc_chrdev_region failed\n");
      return err;
   }

   cdev_init(&fpga_usr.cdev, &fops);
   err = cdev_add(&fpga_usr.cdev, devt, 1);
   if (err)
      goto out_chrdev_region;

   err = class_register(&fpga_class);
   if (err)
      goto out_cdev;

   fpga_usr.dev = device_create(&fpga_class, pdev,
                                devt, NULL, "fpga_usr");
   if (IS_ERR(fpga_usr.dev)) {
      err = PTR_ERR(fpga_usr.dev);
      goto out_class;
   }

   fpga_usr.sysfs_rp3 = sysfs_get_dirent(fpga_usr.dev->kobj.sd, NULL, "rp3_alarm");
   if (!fpga_usr.sysfs_rp3) {
      dev_err(fpga_usr.dev, "sysfs_get_dirent failed for rp3_alarm\n");
      err = -ENOMEM;
      goto out_device;
   }

   fpga_usr.sysfs_refab = sysfs_get_dirent(fpga_usr.dev->kobj.sd, NULL, "refab_alarm");
   if (!fpga_usr.sysfs_refab) {
      dev_err(fpga_usr.dev, "sysfs_get_dirent failed for refab_alarm\n");
      err = -ENOMEM;
      goto out_sysfs_rp3;
   }

   return 0;

out_sysfs_rp3:
   sysfs_put(fpga_usr.sysfs_rp3);
out_device:
   device_unregister(fpga_usr.dev);
out_class:
   class_unregister(&fpga_class);
out_cdev:
   cdev_del(&fpga_usr.cdev);
out_chrdev_region:
   unregister_chrdev_region(devt, 1);

   return err;
}

/**
 * @internal
 * @brief Called when the module is removed from the kernel.
 */
int fpga_usr_remove(struct device *pdev)
{
   dev_t devt = fpga_usr.dev->devt;

   sysfs_put(fpga_usr.sysfs_rp3);
   sysfs_put(fpga_usr.sysfs_refab);
   device_unregister(fpga_usr.dev);
   class_unregister(&fpga_class);
   cdev_del(&fpga_usr.cdev);
   unregister_chrdev_region(devt, 1);

   return 0;
}
