/**
*******************************************************************************
* @file                  AaSysComNid.h
* @version               Draft
* @date                  15-Mar-2013
* @author                RT-CC&S Team
*
* @brief                 NIDs definition
* @module                AaSysComNid
* @owner                 RT-CC&S Team
*
* Copyright 2013-2014 Nokia Solutions and Networks. All rights reserved.
*******************************************************************************/

#ifndef IFAASYSCOM_NID_H
#define IFAASYSCOM_NID_H

//#include <inttypes.h>

/*----------------------- DOXYGEN GROUPS -------------------------------------*/

/**
* @defgroup dgAaSysComNid AaSysCom NID definitions
* @ingroup dgAaSysComNid
*
* NID definitions and common NID macros.
*
* This file defines the NIDs for the LRC and other platforms.
* Note that NIDs are product specific.
* Refer to the SysCom documentation for the definition of SICAD and NID
* http://wikis.inside.nokiasiemensnetworks.com/bin/view/CSSWiki/AaSysCom
*
*/
/*@{*/

#if defined(LRC_NID) || defined(FZM_NID)
/* The following defines apply to the Liquid Radio Cloud and FZM platforms */

/**
********************************************************************************
*
* @brief        Set the board portion of a NID. First 8 bits are used to store
*               the slot/logical id
*
* @param[in]    BoardId - Physical Slot/Logical id (1 .. 255).
*               0 is reserved for SysCom.
*
* @return       Formatted BoardId for the NID portion of a SICAD
*
*******************************************************************************/
#define NIDFmtBoard( nid_board ) ((((uint16_t)nid_board) << 8) & 0xFF00)

/**
********************************************************************************
*
* @brief        Set the SoC/Class portion of a NID. 3 bits are used to store the
*               SoC/Logical Class id
*
* @param[in]    SoCId - SoC id/Logical Class (1 .. 7)
*               0 is reserved for SysCom.
*
* @return       Formatted SoC Id for the NID portion of a SICAD
*
*******************************************************************************/
#define NIDFmtSoC( nid_soc ) ((((uint16_t)nid_soc << 5)) & 0x00E0)

/**
********************************************************************************
*
* @brief        Set the Core/Logical Entity portion of a NID. 5 bits are used
*               to store the Core/Logical Entity id.
*               0 is reserved for SysCom.
*
* @param[in]    CoreId - Core/Logical Entity id (1..31)
*
* @return       Formatted Core/Logical Entity Id for the NID portion of a SICAD
*
*******************************************************************************/
#define NIDFmtCore( nid_core ) (((uint16_t)nid_core) & 0x001F)

/**
********************************************************************************
*
* @brief        Format a NID address using slot/SoC/Core info
*
* @param[in]    BoardId  - Physical Slot/Logical id (1 .. 255).
* @param[in]    SoCId    - SoC id/Logical Class (1 .. 7)
* @param[in]    CoreId   - Core/Logical Entity id (1..31)
*
* @return       Formatted NID of a SICAD
*
*******************************************************************************/
#define NIDFmtNid( nid_slot, nid_soc, nid_core ) (NIDFmtBoard(nid_slot)|NIDFmtSoC(nid_soc)|NIDFmtCore(nid_core))

/**
********************************************************************************
*
* @brief        Get the Board Id from a NID.
*
* @param[in]    NID
*
* @return       Physical Slot/Logical id
*
*******************************************************************************/
#define NIDGetBoard( nid_nid ) ((((uint16_t)nid_nid) >> 8) & 0x00FF)

/*Common Reserved NIDs */
#define AASYSCOM_NODE_NONE 0x0000
#define AASYSCOM_NODE_INVALID 0x0010
#define AASYSCOM_NODE_ANY 0x001F
#define AASYSCOM_NODE_LOCAL 0x0001


/*Kepler-2 SoC 1 */
#define NIDKepler2_SoC1   0x01
/*Kepler-2 SoC 2 */
#define NIDKepler2_SoC2   0x02
/*Kepler-2 SoC 3 */
#define NIDKepler2_SoC3   0x03

/*LSI AXM5516 */
#define NIDAXM5516SoC     0x04

/*Kepler 2 - Reserved */
#define NIDK2Reserved     0x00

/*Kepler 2 ARM Eevent Machine Group 1 */
#define NIDK2ARM_EMGRP1   0x11

/*Kepler 2 Process SMP-1 for ARM*/
#define NIDK2ARM_SMP1     0x1F

/*Kepler 2 DSP Core 1 */
#define NIDDSPCore1       0x01
/*Kepler 2 DSP Core 2 */
#define NIDDSPCore2       0x02
/*Kepler 2 DSP Core 3 */
#define NIDDSPCore3       0x03
/*Kepler 2 DSP Core 4 */
#define NIDDSPCore4       0x04
/*Kepler 2 DSP Core 5 */
#define NIDDSPCore5       0x05
/*Kepler 2 DSP Core 6 */
#define NIDDSPCore6       0x06
/*Kepler 2 DSP Core 7 */
#define NIDDSPCore7       0x07
/*Kepler 2 DSP Core 8 */
#define NIDDSPCore8       0x08

/*LSI - Reserved */
#define NIDLSIReserved    0x00

/*LSI ARM Eevent Machine Group 1 */
#define NIDAXM5516_EMGRP1 0x11

/*LSI Process SMP-1 */
#define NIDAXM5516_SMP1   0x1F

/* LSP Slot 1 Definitions */

/*LSP Slot 1 Kepler2 #1 Event Machine Group 1 - ARM cluster 1*/
#define LSP_01_K2_01_EMGRP1 NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2ARM_EMGRP1 )
/*LSP Slot 1 Kepler2 #1 SMP Group 1 - ARM cluster 1*/
#define LSP_01_K2_01_SMP1  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2ARM_SMP1 )

/*LSP Slot 1 Kepler2 #1 DSP 1-8 */
#define LSP_01_K2_01_D01  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore1 )
#define LSP_01_K2_01_D02  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore2 )
#define LSP_01_K2_01_D03  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore3 )
#define LSP_01_K2_01_D04  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore4 )
#define LSP_01_K2_01_D05  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore5 )
#define LSP_01_K2_01_D06  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore6 )
#define LSP_01_K2_01_D07  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore7 )
#define LSP_01_K2_01_D08  NIDFmtNid( 1, NIDKepler2_SoC1, NIDK2DSPCore8 )

/*LSP Slot 1 Kepler2 #2 Event Machine Group 1 - ARM cluster 1*/
#define LSP_01_K2_02_EMGRP1 NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2ARM_EMGRP1 )
/*LSP Slot 1 Kepler2 #2 SMP Group 1 - ARM cluster 1*/
#define LSP_01_K2_02_SMP1  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2ARM_SMP1 )

/*LSP Slot 1 Kepler2 #2 DSP 1-8 */
#define LSP_01_K2_02_D01  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore1 )
#define LSP_01_K2_02_D02  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore2 )
#define LSP_01_K2_02_D03  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore3 )
#define LSP_01_K2_02_D04  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore4 )
#define LSP_01_K2_02_D05  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore5 )
#define LSP_01_K2_02_D06  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore6 )
#define LSP_01_K2_02_D07  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore7 )
#define LSP_01_K2_02_D08  NIDFmtNid( 1, NIDKepler2_SoC2, NIDK2DSPCore8 )

/*LSP Slot 1 Kepler2 #3 Event Machine Group 1 - ARM cluster 1*/
#define LSP_01_K2_03_EMGRP1 NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2ARM_EMGRP1 )
/*LSP Slot 1 Kepler2 #3 SMP Group 1 - ARM cluster 1*/
#define LSP_01_K2_03_SMP1  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2ARMSMP1 )

/*LSP Slot 1 Kepler2 #3 DSP 1-8 */
#define LSP_01_K2_03_D01  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore1 )
#define LSP_01_K2_03_D02  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore2 )
#define LSP_01_K2_03_D03  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore3 )
#define LSP_01_K2_03_D04  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore4 )
#define LSP_01_K2_03_D05  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore5 )
#define LSP_01_K2_03_D06  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore6 )
#define LSP_01_K2_03_D07  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore7 )
#define LSP_01_K2_03_D08  NIDFmtNid( 1, NIDKepler2_SoC3, NIDK2DSPCore8 )

/*LSP Slot 1 LSI Event Machine Group 1 - ARM clusters 1-4*/
#define LSP_01_LSI_EMGRP1 NIDFmtNid( 1, NIDAXM5516SoC, NIDAXM5516_EMGRP1 )
/*LSP Slot 1 LSI SMP Group 1 - ARM clusters 1-4*/
#define LSP_01_LSI_SMP1  NIDFmtNid( 1, NIDAXM5516SoC, NIDAXM5516_SMP1 )


/* LSP Slot 2 Definitions */
//...
/* LSP Slot 16 Definitions */
//...


/*SCP Process SMP-1 for ARM*/
#define NIDSCP_SMP1     0x1F

/*Active SCP*/
#define NIDSCP_ACTIVE_BOARDID   0xFF

/* SCP Slot 20 SoC #1 and 2 Definitions */
#define SCP_20_01_SMP1 NIDFmtNid( 20, 1, NIDSCP_SMP1 )
#define SCP_20_02_SMP1 NIDFmtNid( 20, 2, NIDSCP_SMP1 )

/* SCP Slot 21 SoC #1 and 2 Definitions */
#define SCP_21_01_SMP1 NIDFmtNid( 21, 1, NIDSCP_SMP1 )
#define SCP_21_02_SMP1 NIDFmtNid( 21, 2, NIDSCP_SMP1 )

/* SCP Active SoC #1 and 2 Definitions */
#define SCP_ACTIVE_01_SMP1 NIDFmtNid( NIDSCP_ACTIVE_BOARDID, 1, NIDSCP_SMP1 )
#define SCP_ACTIVE_02_SMP1 NIDFmtNid( NIDSCP_ACTIVE_BOARDID, 2, NIDSCP_SMP1 )


/*AXM 3G*/
/*Active AXM LSP*/
#define NIDAXM_ACTIVE_BOARDID   0xFD

/*LSP -3G AXM Active Event Machine Group 1 - LSI */
#define LSP_3G_AXM_ACTIVE_EMGRP1 NIDFmtNid( NIDAXM_ACTIVE_BOARDID, NIDAXM5516SoC, NIDAXM5516_EMGRP1 )
/*LSP -3G AXM Active SMP Group 1 - LSI */
#define LSP_3G_AXM_ACTIVE_SMP1  NIDFmtNid( NIDAXM_ACTIVE_BOARDID, NIDAXM5516SoC, NIDK2LSI_SMP1 )


#endif /*LRC&FZM NID */

#endif /*_IFAASYSCOM_NID_H*/
