/*
 * Copyright (c) 2011-2014 Nokia Solutions and Networks
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
/**
 * @file cpld-usr.h
 * @author Nokia Solutions and Networks
 * @remarks Copyright (C) 2011-2014 Nokia Solutions and Networks
 *
 * @par Purpose
 *
 * Defines APIs for accessing the CPLD via user-space on FZM.
 *
 * @par Proc files
 * @li @c /proc/cpld-usr - Reading this file will output the contents of the
 *                          readable CPLD registers.
 *
 * @par Linux device
 * @li @c /dev/fzm/cpld-usr
 *
 * @par IOCTL Commands
 * @li @c CPLD_RESET_DEVICE - Puts the specified device into reset
 * @li @c CPLD_RELEASE_RESET_DEVICE - Releases the specified device from reset
 * @li @c CPLD_RD_RESET_STATUS - Reads the RRR2 register for reset status
 * @li @c CPLD_SET_LED - Sets the Status, BH, or RF LED color and blink state
 * @li @c CPLD_GET_LED - Gets the Status, BH, or RF LED color and blink state
 * @li @c CPLD_GET_SCR5 - Gets the scratchpad 5 register value.
 * @li @c CPLD_SET_SCR5 - Sets the scratchpad 5 register value.
 * @li @c CPLD_TOGGLE_SCR5 - Toggles bits in the scratchpad 5 register.
 *
 * @par CPLD_RESET_DEVICE(Cpld_device_t device)
 * @code
 * ioctl(fd, CPLD_RESET_DEVICE, CPLD_RESET_PSE);
 * @endcode
 *
 * @par CPLD_RELEASE_RESET_DEVICE(Cpld_device_t device)
 * @code
 * ioctl(fd, CPLD_RELEASE_RESET_DEVICE, CPLD_RESET_PSE);
 * @endcode
 *
 * @par CPLD_RD_RESET_STATUS(uint32_t *status)
 * @code
 * uint32_t status;
 * ioctl(fd, CPLD_RD_RESET_STATUS, &status);
 * if (status & CPLD_RRR2_BIT_PSE) // PSE is out of reset
 * @endcode
 *
 * @par CPLD_SET_LED(struct cpld_led *)
 * @code
 * struct cpld_led led;
 * memset(&led, 0, sizeof(led));
 * // turn Status LED green and make it blink
 * led.led = CPLD_LED_STATUS;
 * led.green = 1;
 * led.blink = 1;
 * ioctl(fd, CPLD_SET_LED, &led);
 * @endcode
 *
 * @par CPLD_GET_LED(struct cpld_led *)
 * @code
 * struct cpld_led led;
 * memset(&led, 0, sizeof(led));
 *
 * // Get backhaul LED color and blink state
 * led.led = CPLD_LED_BH;
 * ioctl(fd, CPLD_GET_LED, &led);
 * if (led.green)
 *   printf("BH led is green\n");
 * if (led.red)
 *   printf("BH led is red\n");
 * if (led.blink)
 *   printf("BH led is set to blink\n");
 * @endcode
 *
 * @par CPLD_GET_SCR5(uint8_t *)
 * @code
 * uint8_t *val;
 * ioctl(fd, CPLD_GET_SCR5, &val);
 * printf("SCR5 val is %#x\n", val);
 * @endcode
 *
 * @par CPLD_SET_SCR5(uint8_t)
 * @code
 * ioctl(fd, CPLD_SET_SCR5, 0x4);
 * @endcode
 *
 * @par CPLD_TOGGLE_SCR5(struct cpld_toggle_scr5 *)
 * @code
 * struct cpld_toggle_scr5 tog;
 * tog.turn_on = 0x40;
 * tog.turn_off = 0x80;
 * ioctl(fd, CPLD_TOGGLE_SCR5, &tog);
 * @endcode
 *
 * @par Netlink Notifications
 * TBD
 *
 * @par Reset Attributes
 * Certain devices can be reset through sysfs files at
 * /sys/devices/virtual/cpld-usr/cpld-usr/[*]_reset.  Write a 1 to put in
 * reset and a 0 to take out of reset.
 *
 * @par LED access/usage
 * Status, backhaul, and RF LED states may optionally be read/written through
 * sysfs at /sys/devices/virtual/cpld-usr/cpld-usr/[*]_led (st_led, bh_led,
 * and rf_led).  Reading any of these files will display the current state for
 * the associated LED (e.g. "blinking green", "blinking red", "solid green",
 * "solid red", "off").  Write the character 'o' to the file to turn the LED
 * off; write 'r' to turn the LED red; write 'g' to turn it green; write 'y'
 * to turn both red/green on.  Writing these characters alone will set the LED
 * to a solid color; writing them with a 'b' suffix will blink that color.
 * For example, "echo gb > st_led" will set the status LED to blinking green.
 *
 * @par Scratchpad access/usage
 * The scratchpad registers on the CPLD can be read/written through sysfs at
 * /sys/devices/virtual/cpld-usr/cpld-usr/scr* .  The values read will be in
 * hex.  These can be written in any format recognized by strtoul (decimal,
 * hex, or octal).  SCR3 is read-only and writeable only by the PHOST.  Bits
 * 0x3 are used by the PHOST bootloader.  SCR5 bit 0x04 is reserved for use
 * by PHOST HWTC.  SCR2 bits 0x7 are reserved for use for boot failure
 * detection.
 *
 * <center>
 * <table>
 *   <tr> <th colspan="2">SCR1 - SoC Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td>Unused</td> </tr>
 *   <tr> <td>1</td> <td>Unused</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>Unused</td> </tr>
 *   <tr> <td>5</td> <td>Unused</td> </tr>
 *   <tr> <td>6</td> <td>Unused</td> </tr>
 *   <tr> <td>7</td> <td>Unused</td> </tr>
 * </table>
 *
 * <br>
 *
 * <table>
 *   <tr> <th colspan="2">SCR2 - SoC Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td>Unused</td> </tr>
 *   <tr> <td>1</td> <td>Unused</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>DAN Enet Ready Indication (read by Cavium)</td> </tr>
 *   <tr> <td>5</td> <td rowspan="3">DAN boot failure detection</td> </tr>
 *   <tr> <td>6</td>
 *   <tr> <td>7</td>
 * </table>
 *
 * <br>
 *
 * <table>
 *   <tr> <th colspan="2">SCR3 - Phost Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td rowspan="2">CRTM u-boot selection</td> </tr>
 *   <tr> <td>1</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>Unused</td> </tr>
 *   <tr> <td>5</td> <td rowspan="3">Cav boot failure detection</td> </tr>
 *   <tr> <td>6</td> </tr>
 *   <tr> <td>7</td> </tr>
 * </table>
 *
 * <br>
 *
 * <table>
 *   <tr> <th colspan="2">SCR4 - SoC Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td>Unused</td> </tr>
 *   <tr> <td>1</td> <td>Unused</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>Unused</td> </tr>
 *   <tr> <td>5</td> <td>Unused</td> </tr>
 *   <tr> <td>6</td> <td>Unused</td> </tr>
 *   <tr> <td>7</td> <td>Unused</td> </tr>
 * </table>
 *
 * <br>
 *
 * <table>
 *   <tr> <th colspan="2">SCR5 - Both Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td>Unused</td> </tr>
 *   <tr> <td>1</td> <td>Unused</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>Unused</td> </tr>
 *   <tr> <td>5</td> <td>Unused</td> </tr>
 *   <tr> <td>6</td> <td>Unused</td> </tr>
 *   <tr> <td>7</td> <td>Unused</td> </tr>
 * </table>
 *
 * <br>
 *
 * <table>
 *   <tr> <th colspan="2">SCR6 - Both Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td>Unused</td> </tr>
 *   <tr> <td>1</td> <td>Unused</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>Unused</td> </tr>
 *   <tr> <td>5</td> <td>Unused</td> </tr>
 *   <tr> <td>6</td> <td>Unused</td> </tr>
 *   <tr> <td>7</td> <td>Unused</td> </tr>
 * </table>
 *
 * <br>
 *
 * <table>
 *   <tr> <th colspan="2">SCR7 - Both Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td>Unused</td> </tr>
 *   <tr> <td>1</td> <td>Unused</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>Unused</td> </tr>
 *   <tr> <td>5</td> <td>Unused</td> </tr>
 *   <tr> <td>6</td> <td>Unused</td> </tr>
 *   <tr> <td>7</td> <td>Unused</td> </tr>
 * </table>
 *
 * <br>
 *
 * <table>
 *   <tr> <th colspan="2">SCR8 - Both Writable</th> </tr>
 *   <tr> <th>Bit</th> <th>Description</th> </tr>
 *   <tr> <td>0 (MSB)</td> <td>Unused</td> </tr>
 *   <tr> <td>1</td> <td>Unused</td> </tr>
 *   <tr> <td>2</td> <td>Unused</td> </tr>
 *   <tr> <td>3</td> <td>Unused</td> </tr>
 *   <tr> <td>4</td> <td>Unused</td> </tr>
 *   <tr> <td>5</td> <td>Unused</td> </tr>
 *   <tr> <td>6</td> <td>Unused</td> </tr>
 *   <tr> <td>7</td> <td>Unused</td> </tr>
 * </table>
 *
 * </center>
 *
 * @par
 * <em>
 * Soft kitty, warm kitty, little ball of fur.<br/>
 * Happy kitty, sleepy kitty, purr purr purr...
 * </em>
 *
 * @par REVISIONS
 * <pre>
 * Date       Who       CR         Description
 * ---------- --------- ---------- --------------------------------------------
 * 07/27/2011 NSN       1426679    Created.
 * 11/02/2011 qdpx86    1448692    Added attributes for SWSTAT bits
 * 01/31/2012 tberger1  1458932    Updated for krypton
 * 04/11/2013 bbausch1             Porting to FZM
 * 11/04/2013 marerosa             Added support for more SFP Digital Diagnostic reads
 * </pre>
 */

#ifndef CPLD_USR_H
#define CPLD_USR_H

#include <linux/types.h>

#ifdef __KERNEL__
#  include "fzm_cpld.h"
#endif

#include "fzm_ioctls.h"

//shared kernel and userspace

/****************************************************************************
 * Enumerations
 ***************************************************************************/

/**
 * This is used by the reset/release-reset IOCTLs.
 */
typedef enum {
   CPLD_RESET_BH_PHY = 1,         /**< Backhaul PHY reset, not self clearing. */
   CPLD_RESET_BH_PHY2,
   CPLD_RESET_BH_PHY3,
   CPLD_RESET_FPGA,               /**< FPGA reset, not self clearing. */
   CPLD_RESET_PSE,                /**< PSE reset, not self clearing. */
} Cpld_device_t;

/**
 * The different available LEDs.
 */
typedef enum {
   CPLD_LED_STATUS = 0,          /**< Status LED */
   CPLD_LED_BH = 1,              /**< BH LED */
   CPLD_LED_RF = 2,              /**< RF LED */
   CPLD_LED_OFF = 3,             /**< ALL OFF LED */
} Cpld_led_t;


/**
 * The different available sfp attributes.
 */
typedef enum {
   SFP_VENDOR_NAME = 0,          /**< Vendor name */
   SFP_POWER = 1,              /**< Power */
   SFP_TX_VALUE = 2,              /**< TX_VAL */
   SFP_RATE_SEL = 3,             /**< RATE_SEL */
   SFP_DIAG_MONITOR_TYPE = 4,    /**< DIAG_MONITOR_TYPE */
   SFP_CONNECTOR_VALUE = 5,
   SFP_BR_NOMINAL = 6,
   SFP_BR_MAX = 7,
   SFP_BR_MIN = 8,
   SFP_TRANCEIVER_CODE = 9,
   SFP_PART_NUMBER = 10,
   SFP_WAVE_LENGTH = 11,
   SFP_SERIAL_NUMBER = 12,
   SFP_VENDOR_REVISION = 13,
   SFP_SFF_8472_COMPLIANCE = 14,
   SFP_DATE_CODE = 15
} Sfp_attrs_t;

/**
 * The different available sfp attributes.
 */
typedef enum {
   SFP_DIAG_TEMP = 0,          /**< DIAG TEMP */
   SFP_DIAG_TSLOPE = 1,
   SFP_DIAG_TOFFSET = 2,
   SFP_DIAG_VOLTAGE = 3,
   SFP_DIAG_VSLOPE = 4,
   SFP_DIAG_VOFFSET = 5,
   SFP_DIAG_CURRENT = 6,
   SFP_DIAG_ISLOPE = 7,
   SFP_DIAG_IOFFSET = 8,
   SFP_DIAG_TX_POWER = 9,
   SFP_DIAG_TX_PWR_SLOPE = 10,
   SFP_DIAG_TX_PWR_OFFSET = 11,
   SFP_DIAG_RX_POWER = 12,
   SFP_DIAG_RX_PWR_0 = 13,
   SFP_DIAG_RX_PWR_1 = 14,
   SFP_DIAG_RX_PWR_2 = 15,
   SFP_DIAG_RX_PWR_3 = 16,
   SFP_DIAG_RX_PWR_4 = 17,
   SFP_DIAG_EXT_STATUS_CTRL = 18,
   SFP_DIAG_STATUS_CTRL = 19,
   SFP_DIAG_ALARM_WARNING_BYTES

} Sfp_diag_attrs_t;
/**
 * The different states of an individual LED.
 */
struct cpld_led {
   Cpld_led_t led;               /**< Which LED */
   unsigned red: 1;              /**< Red LED eluminate */
   unsigned green: 1;            /**< Green LED eluminate */
   unsigned blink: 1;            /**< Whether or not it is blinking */
};

/**
 * @brief The structure used to toggle bits in CPLD SCR5
 * The member turn_on represents bits to be toggled to on.  The
 * member turn_off represents bits to be toggled to off.
 */
struct cpld_toggle_scr5 {
   __u8 turn_on;      /**< Bits to turn on */
   __u8 turn_off;     /**< Bits to turn off */
};

/**
 * @brief The struct used to set or get the I2CC registers in the CPLD.
 */
struct cpld_sfp_serial_write {
   Sfp_attrs_t attr;
   __u8 regs;  /**< I2CC3 register value */
};

/**
 * @brief The struct used to set or get the I2CC registers in the CPLD.
 */
struct cpld_sfp_serial_read {
   Sfp_attrs_t attr;
   __u8 regs[17];  /**< I2CC3 register value */
};

/**
 * @brief The struct used to set or get the I2CC registers in the CPLD.
 */
struct cpld_sfp_diag_write {
   Sfp_diag_attrs_t diagAttr;
   __u8 regs;  /**< I2CC3 register value */
};

/**
 * @brief The struct used to set or get the I2CC registers in the CPLD.
 */
struct cpld_sfp_diag_read {
   Sfp_diag_attrs_t diagAttr; /**< I2CC3 register value */
   __u8 regs[4];  /* regs 1 would be most significant bit */
};

struct cpld_sfpc_read {
   __u8 regs;
};

struct cpld_sfpc_write {
   __u8 regs;
};

struct cpld_ioctl_rw {
   __u32 offset;
   __u8 value;
};

#endif /* CPLD_USR_H */
