/**
 * @file msg_rtr_arm.h
 */
 

#ifndef __MSGRTRARM_DRIVER_H__
#define __MSGRTRARM_DRIVER_H__

#include <linux/skbuff.h>
/**
* @brief   The callback type.  
*
*/
typedef void (*msgRtrARMCb)(struct sk_buff *);

/**
* @brief   The callback gives you a socket buffer for a previously registered NID.  
*
* <b> Prototype: </b>
*  The following is the C prototype for the expected MsgRtrARM API.  The callback
*  gives you an address to a socket buffer for a registered SysCom Nid.  The socket buffer 
*  is guaranteed to have a minimum of 32 bytes headroom to allow the addition or 
*  manipulation of any network headers. Once this callback is invoked, the application
*  is responsible for freeing the sk_buff. 
*
*  @verbatim
      void MsgRtrARMCallback(sk_buff * buffer)
   @endverbatim
*
*  <b> Parameter </b>
*  @n  sk_buff    - a socket buffer address 
*
*  <b> Return Value </b>
*  @n  Not Applicable
*/
void MsgRtrARMCallback(struct sk_buff * buffer);


/**
* @brief   This function is used to register a callback associated with 
* a SICAD.
*
* <b> Prototype: </b>
*  The following is the C prototype for the expected MsgRtrARM API. This function 
*  registers a callback to a corresponding SysCom Nid within the MsgRtrARM service.  
*  Note, duplicate registrations will not return error, but will simply overwrite
*  the existing callback associated with the passed in Nid.  There is only one 
*  Nid registration allowed.  Multiple Nid registrations are currently not 
*  supported, and a registration for a second unique Nid will return failure. 
*
*  @verbatim
      int MsgRtrARMRegisterSysComID(TAaSysComNid sysComNid, msgRtrARMCb* callback)
   @endverbatim
*
*  <b> Parameter </b>
*  @n  sysComNid      - SysCom NID
*  @n  msgRtrARMCb    - callback address for when a message is received
*
*  <b> Return Value </b>
*  @n  0 - success
*  @n  negative value on failure
*/
//int MsgRtrARMRegisterSysComID(TAaSysComNid sysComNid, msgRtrARMCb* callback);
int MsgRtrARMRegisterSysComID(u16 sysComNid, msgRtrARMCb callback);

/**
* @brief   This function unregisters a SICAD.
*
* <b> Prototype: </b>
*  The following is the C prototype for the expected MsgRtrARM API.
*
*  @verbatim
      int MsgRtrARMUnregister(TAaSysComNid sysComNid)
   @endverbatim
*
*  <b> Parameter </b>
*  @n  sysComNid    - SysCom NID to be unregistered 
*
*  <b> Return Value </b>
*  @n  0 - success
*  @n  negative value on failure
*/
//int MsgRtrARMUnregister(TAaSysComNid sysComNid);
int MsgRtrARMUnregister(u16 sysComNid);

/**
* @brief   This function sends a message to a DSP core.
*
* <b> Prototype: </b>
*  The following is the C prototype for the expected MsgRtrARM API.  It is assumed that
*  the Syscom header is at at the top of the data portion of the socket buffer.  If an 
*  error occurs, it is up to the application to free the buffer.  Otherwise the socket
*  buffer is freed when the DMA transmit completes.
*
*  @verbatim
      int MsgRtrARMSend(sk_buff* buffer)
   @endverbatim
*
*  <b> Parameter </b>
*  @n  buffer    - Buffer to be sent to the DSP core. 
*
*  <b> Return Value </b>
*  @n  0 - success
*  @n  negative value on failure
*/
int MsgRtrARMSend(struct sk_buff* buffer);

/**
* @brief   This function sends a "loop back" message.  
*
* <b> Prototype: </b>
*  The following is the C prototype for the expected MsgRtrARM API.  The message will be
*  sent via PKT_DMA to the receive flow of MsgRtrArm.  It will then be subject to normal
*  classification and routing as though it was a normal SysCom message from the DSP.  If 
*  the NID destination matches a registered NID, the appropriate callback will be 
*  called, otherwise the packet will be sent to Linux.  It is assumed that
*  the Syscom header is at at the top of the data portion of the socket buffer.  If an 
*  error occurs, it is up to the application to free the buffer.  Otherwise the socket
*  buffer is freed when the DMA transmit completes.  The source and destination SICAD
*  addresses are swapped prior to the DMA for loopback.  The source SICAD in the buffer 
*  should match the sender's registered NID in order for the callback to be called.
*
*  @verbatim
      int MsgRtrARMSend_loopback(sk_buff* buffer)
   @endverbatim
*
*  <b> Parameter </b>
*  @n  buffer    - Buffer to be sent via loopback. 
*
*  <b> Return Value </b>
*  @n  0 - success
*  @n  negative value on failure
*/
int MsgRtrARMSend_loopback(struct sk_buff* buffer);

#endif
